// -*- C++ -*-
/*!
 * @file ExecutionContext.h
 * @brief ExecutionContext class
 * @date $Date: 2007-01-21 13:43:18 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ExecutionContext.h 1225 2009-02-28 02:30:25Z n-ando $
 *
 */

#ifndef ExecutionContext_h
#define ExecutionContext_h

#include <rtm/idl/RTCSkel.h>
#include <rtm/SystemLogger.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class ExecutionContextBase
   * @brief ExecutionContext base class
   *
   * ExecutionContext  RTC ̃rWlXWbNsXbh
   * 番邱Ƃ\ɂB ReLXg͘_IȃXbhɑ
   * s RTC ɑ΂Ē񋟂BReLXg͎X̃Iy[V
   * ꎩgуReLXgɏ RTC Ԃ₢킹ύX
   * 肷邱ƂłB
   * ֌W͈̕ȉ̓̗RɋdvłB
   * <ul>
   * <li> ̃R|[lg͈̃m[h̓vZXŖڂɋ
   *      \܂B ꂼ̃R|[lggXbh
   *      ꍇAXbhꎩg̐Xbh݂𓯊邽߂ɁA
   *      ~hEGA̓A^CAvP[VɂAA^C
   *      _Iȗv𖞂ƂłȂ\܂B
   * <li> ̃AvP[V͈قȂŎs镡̓Ɨ^XN
   *      s邩܂BƂ΁AZT̃TvO𑁂
   *      sȂA[UC^[tF[X̍XVxōs
   *      ܂B
   * </ul>
   *
   * <b> I[i[VbvƃReLXgւ̏ </b><br>
   * e ExecutionContext ͈ RTC ɏĂA RTC сA
   *  RTC ܂ RTC ̎s𒼐ړI܂͔񒼐ړIɍsB 
   * ȏ ExecutionContext  RTC  autonomous RTC ƌĂ΂B
   * autonomous RTC тɊ܂܂邻̃TuZbg RTC (̓AvP[
   * VJ҂ɂ`B)  ExecutionKind ɏ]āA~hEGA
   * ɂsBExecutionKind ͂ꂼ RTC ̃Iy[V́A
   * ǂԂŎs邩K肷B RTC ͂̃ReLXg
   * uvƕ\Bp\ ExecutionKind ͈ȉŏqׂB
   * ʓIɂ́ARTC ExecutionContext ̊֌W͑Αł邩ȂB
   * Ƃ΁A RTC  ExecutionContext ɂsꂽA
   *  RTC  ExecutionContext sꂽ肷邩ȂB
   * RTC̃ReLXgsꍇɂ́AExecutionContext 
   * X^[g邢̓Xgbv肷邱ƂɂASẴR|[lg
   * ̃CtTCNԂJڂ\B
   *
   * <b> Logical and Physical Threads </b><br>
   * ExecutionContext _IXbh\Ă̂́A
   * ǂċ̓IȃXbhɃ}bsO邩ƂÍA
   * AvP[VfvCgɈς˂ĂB
   * ɍۂĂ͊֘AtExecutionContextƋ̓IȃXbhΈ
   * Ń}bsOA̃Xbh ExecutionContext Ɋ֘At
   * ꂽ肷\B
   * RTC  ExecutionContext ɂsꍇ̃RJV[Ǘ
   * ͎ˑłB
   *
   * @else
   *
   * @class ExecutionContextBase
   * @brief ExecutionContext base class
   *
   * An ExecutionContext allows the business logic of an RTC to be decoupled
   * from the thread of control in which it is executed. The context represents
   * a logical thread of control and is provided to RTCs at runtime as an
   * argument to various operations, allowing them to query and modify their
   * own state, and that of other RTCs executing within the same context, in
   * the lifecycle.
   * This separation of concerns is important for two primary reasons:
   * <ul>
   * <li> Large number of components may collaborate tightly within a single
   *      node or process. If each component were to run within its own thread
   *      of control, the infrastructure may not be able to satisfy the
   *      timeliness and determinism requirements of real-time applications due
   *      to the large number of threads and the required synchronization
   *      between them.
   * <li> A single application may carry out a number of independent tasks that
   *      require different execution rates. For example, it may need to sample
   *      a sensor periodically at a very high rate and update a user
   *      interface at a much lower rate.
   * </ul>
   *
   * <b> Ownership and Participation </b><br>
   * Each execution context is owned by a single RTC and may be used to execute
   * that RTC and the RTCs contained within it, directly or indirectly. An RTC
   * that owns one or more execution contexts is known as an autonomous RTC.
   * An autonomous RTC and some subset of the RTCs within it (to be defined by
   * the application developer) shall be executed by the infrastructure
   * according to the context's execution kind, which defines when each RTC's
   * operations will be invoked when and in which order. These RTCs are said
   * to participate in the context. The available execution kinds are described
   * below.
   * The relationship between RTCs and execution contexts may be many-to-many
   * in the general case: multiple RTCs may be invoked from the same execution
   * context, and a single RTC may be invoked from multiple contexts.
   * In the case where multiple RTCs are invoked from the same context,
   * starting or stopping the context shall result in the corresponding
   * lifecycle transitions for all of those components.
   *
   * <b> Logical and Physical Threads </b><br>
   * Although an execution context represents a logical thread of control,
   * the choice of how it maps to a physical thread shall be left to the
   * applicationfs deployment environment. Implementations may elect to
   * associate contexts with threads with a one-to-one mapping, to serve
   * multiple contexts from a single thread, or by any other means. In the
   * case where a given RTC may be invoked from multiple contexts, concurrency
   * management is implementation-dependent.
   *
   * @endif
   */
  class ExecutionContextBase
    : public virtual POA_RTC::ExecutionContextService,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     * @else
     * @brief Constructor
     * @endif
     */
    ExecutionContextBase(RTObject_ptr owner);


    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~ExecutionContextBase(void) {};

    //============================================================
    // ExecutionContext interfaces
    //============================================================
    /*!
     * @if jp
     * @brief ExecutionContext sǂ̃eXg
     *
     * <h3> Description </h3>
     * ̃Iy[V͂ ExecutionContext ̎sԂԂB
     *
     * <h3> Semantics </h3>
     * ̃ReLXg Running Ԃł΁ASẴANeBu RTC 
     * QR|[lg̓ReLXg ExecutionKind ɏ]ĎsB
     *
     * @return sԂł true ԂB
     *
     * @else
     *
     * @brief Test for ExecutionContext running state
     *
     * <h3>Description</h3>
     * This operation shall return true if the context is in the Running state.
     *
     * <h3> Semantics </h3>
     * While the context is Running, all Active RTCs participating in the
     * context shall be executed according to the context's execution kind.
     *
     * @return If the ExecutionContext is running, it returns true.
     *
     * @endif
     */
    virtual CORBA::Boolean is_running();


    /*!
     * @if jp
     *
     * @brief ExecutionContext X^[g
     *
     * <h3> Description </h3>
     *  ExecutionContext  Running ԂɑJڂ邽߂̃Iy[VB
     * ԑJڂꍇAComponentAction::on_startup sB
     * (OMG RTC Specification section 2.2.2.5.2 Q)
     *
     * <h3> Semantics </h3>
     * ExecutionContext ɑ΂ start  stop ͌JԂsB
     *
     * <h3> Constraints </h3>
     * @return ̃Iy[V ExecutionContext  Stopped ԂɂȂƂ
     *         sƁAPRECONDITION_NOT_MET ԂB
     *
     * @else
     *
     * @brief Start the ExecutionContext
     *
     * <h3> Description </h3>
     * Request that the context enter the Running state. Once the state
     * transition occurs, the ComponentAction::on_startup operation
     * (see OMG RTC Specification section 2.2.2.5.2) will be invoked.
     *
     * <h3> Semantics </h3>
     * An execution context may be started and stopped multiple times.
     *
     * <h3> Constraints </h3>
     * @return This operation shall fail with PRECONDITION_NOT_MET if the
     *         context is not in the Stopped state.
     *
     * @endif
     */
    virtual ReturnCode_t start();


    /*!
     * @if jp
     *
     * @brief ExecutionContext Xgbv
     *  
     * <h3> Description </h3>
     *  ExecutionContext  Stopped ԂɑJڂ邽߂̃Iy[VB
     * ԑJڂꍇAComponentAction::on_shutdown sB
     * (OMG RTC Specification section 2.2.2.5.4 Q)
     *
     * <h3> Semantics </h3>
     * ExecutionContext ɑ΂ start  stop ͌JԂsB
     *
     * <h3> Constraints </h3>
     * @return ̃Iy[V ExecutionContext  Stopped ԂɂȂƂ
     *         sƁAPRECONDITION_NOT_MET ԂB
     *
     * @else
     * @brief Stop the ExecutionContext
     *
     * <h3> Description </h3>
     * Request that the context enter the Stopped state. Once the transition
     * occurs, the ComponentAction::on_shutdown operation
     * (see OMG RTC Specification section 2.2.2.5.4) will be invoked.
     *
     * <h3> Semantics </h3>
     * An execution context may be started and stopped multiple times.
     *
     * <h3> Constraints </h3>
     * @return This operation shall fail with PRECONDITION_NOT_MET if the
     *         context is not in the Stopped state.
     *
     * @endif
     */
    virtual ReturnCode_t stop();


    /*!
     * @if jp
     *
     * @brief s(Hz)擾
     *
     * <h3> Description </h3>
     * ̃Iy[V͂̃ReLXgɎQĂANeBuRTC
     * ̎sHzŕԂB
     *
     * <h3> Semantics </h3>
     * ɂĂ PERIODIC ȊO ExecutionKind ̎I
     * ͋[Iȏ邱Ƃ܂B̏ꍇɂ́A
     * ̃Iy[V̌ʂ͎`ɏ]܂B
     * AExecutionContextIsȂReLXgłꍇA
     * ̃Iy[V͎s܂B
     *
     * <h3> Constraints </h3>
     *@return ExecutionContext  ExecutionKind  PERIODICł΁A
     *         0傫lԂB
     *
     * @else
     *
     * @brief Get executionrate(Hz)
     *
     * <h3> Description </h3>
     * This operation shall return the rate (in hertz) at which its Active
     * participating RTCs are being invoked.
     *
     * <h3> Semantics </h3>
     * An implementation is permitted to perform some periodic or
     * quasi-periodic processing within an execution context with an
     * ExecutionKind other than PERIODIC. In such a case, the result of this
     * operation is implementation-defined.
     * If no periodic processing of any kind is taking place within the
     * context, this operation shall fail as described in section 2.2.1 above.
     *
     * <h3> Constraints </h3>
     * @return If the context has an ExecutionKind of PERIODIC, this operation
     *         shall return a rate greater than zero.
     *
     * @endif
     */
    virtual CORBA::Double get_rate();


    /*!
     * @if jp
     *
     * @brief s(Hz)^ 
     *
     * <h3> Description </h3>
     * ̃Iy[V͂̃ReLXgANeBuRTCs
     * ^B
     *
     * <h3> Semantics </h3>
     *  ExecutionContext  ExecutionKind  PERIODIC łꍇA
     * s̕ύXɔA ExecutionContext ɎQ RTC Ƃ
     * o^Ă DataFlowComponentAction  RTC ̃Iy[V
     * on_rate_changed sB
     * ɂĂ PERIODIC ȊO ExecutionKind ̎I
     * ͋[Iȏ邱Ƃ܂BĂ̎
     * Ď get_rate ɂ擾łꍇɂ́A^ꂽ
     * LȎł΁ÃIy[V͎ݒ肷̂Ƃ܂B
     * AExecutionContextIsȂReLXgłꍇA
     * ̃Iy[V ReturnCode_t::UNSUPPORTED ԂĎs܂B
     *
     * <h3> Constraints </h3>
     * @return ^0傫Ȃ΂Ȃ炸AłȂꍇ
     *         ̃Iy[V ReturnCode_t::BAD_PARAMETER ԂB
     *
     * @else
     *
     * @brief Set rate (Hz)
     *
     * <h3> Description </h3>
     * This operation shall set the rate (in hertz) at which this context's
     * Active participating RTCs are being called.
     *
     * <h3> Semantics </h3>
     * If the execution kind of the context is PERIODIC, a rate change shall
     * result in the invocation of on_rate_changed on any RTCs realizing
     * DataFlowComponentAction that are registered with any RTCs participating
     * in the context.
     * An implementation is permitted to perform some periodic or
     * quasi-periodic processing within an execution context with an
     * ExecutionKind other than PERIODIC. If such is the case, and the
     * implementation reports a rate from get_rate, this operation shall set
     * that rate successfully provided that the given rate is valid. If no
     * periodic processing of any kind is taking place within the context,
     * this operation shall fail with ReturnCode_t::UNSUPPORTED.
     *
     * <h3> Constraints </h3>
     * @return The given rate must be greater than zero. Otherwise, this
     *         operation shall fail with ReturnCode_t::BAD_PARAMETER.
     *
     * @endif
     */
    virtual ReturnCode_t set_rate(CORBA::Double rate);


    /*!
     * @if jp
     *
     * @brief R|[lgANeBu
     *
     * <h3> Description </h3>
     * ^ RTC ͂̃ReLXgɂ Inactive łA]
     * ExecutionContext  ExecutionKind ɏ]ĎsĂȂB
     * ̃Iy[VɂĂ RTC ͂ ExecutionContext ɂ
     * s Active ԂɑJڂB
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> ExecutionContext ͂ꎩgɎQĂR|[lg Active
     *      łȂ̂ŁA^ꂽRTC̃ReLXgɎQĂ
     *      ΁ÃIy[V BAD_PARAMETER ԂĎsB
     * <li> Error Ԃɂ RTC  reset Ȃ Active łȂB
     *      ^ꂽ RTC  Error ԂɂꍇÃIy[V
     *      PRECONDITION_NOT_MET ԂĎsB
     * </ul>
     * @return OK, BAD_PARAMETER  PRECONDITION_NOT_MET ԂB
     *
     * @else
     *
     * @brief Activate a component
     * 
     * <h3> Description </h3>
     * The given participant RTC is Inactive and is therefore not being invoked
     * according to the execution context's execution kind. This operation
     * shall cause the RTC to transition to the Active state such that it may
     * subsequently be invoked in this execution context.
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> An execution context can only activate its participant components.
     *      If the given RTC is not participating in the execution context,
     *      this operation shall fail with BAD_PARAMETER.
     * <li> An RTC that is in the Error state cannot be activated until after
     *      it has been reset. If the given RTC is in the Error state, this
     *      operation shall fail with PRECONDITION_NOT_MET.
     * </ul>
     * @return This operation returns OK, BAD_PARAMETER or PRECONDITION_NOT_MET
     *
     * @endif
     */
    virtual ReturnCode_t activate_component(LightweightRTObject_ptr comp);


    /*!
     * @if jp
     *
     * @brief R|[lgANeBu
     *
     * <h3> Description </h3>
     * ^ RTC ͂̃ReLXg̒ Active ԂłB
     * ̃Iy[VɂĂ RTC  Inactive ԂɑJڂB
     * Inactive Ԃł Active ܂ RTC ͎sȂB
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> ExecutionContext ͎gɏR|[lĝݔANeBu
     *      łBA^ꂽ RTC  ExecutionContext ɏĂ
     *      Ȃꍇ BAD_PARAMETER ԂĎsB
     * </ul>
     * @return OK, BAD_PARAMETER ԂB
     *
     * @else
     *
     * @brief Deactivate a component
     *
     * <h3> Description </h3>
     * The given RTC is Active in the execution context. Cause it to transition
     * to the Inactive state such that it will not be subsequently invoked from
     * the context unless and until it is activated again.
     *  
     * <h3> Constraints </h3>
     * <ul>
     * <li> An execution context can only deactivate its participant
     *      components. If the given RTC is not participating in the execution
     *      context, this operation shall fail with BAD_PARAMETER.
     * </ul>
     * @return This operation returns OK or BAD_PARAMETER.
     *
     * @endif
     */
    virtual ReturnCode_t deactivate_component(LightweightRTObject_ptr comp);

    /*!
     * @if jp
     *
     * @brief R|[lgZbg
     *
     * <h3> Description </h3>
     * ^ RTC ͂̃ReLXg̒ Error ܂ Active ԂłB
     * ̃Iy[VɂĂ RTC  Inactive ԂɑJڂB
     * Inactive Ԃł Active ܂ RTC ͎sȂB
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> ExecutionContext ͎gɏR|[lĝݔANeBu
     *      łBA^ꂽ RTC  ExecutionContext ɏĂ
     *      Ȃꍇ BAD_PARAMETER ԂĎsB
     * </ul>
     * @return OK, BAD_PARAMETER ԂB
     *
     * @else
     *
     * @brief Deactivate a component
     *
     * <h3> Description </h3>
     * The given RTC is Error or Active in the execution context. Cause it to
     * transition to the Inactive state such that it will not be subsequently
     * invoked from the context unless and until it is activated again.
     *  
     * <h3> Constraints </h3>
     * <ul>
     * <li> An execution context can only deactivate its participant
     *      components. If the given RTC is not participating in the execution
     *      context, this operation shall fail with BAD_PARAMETER.
     * </ul>
     * @return This operation returns OK or BAD_PARAMETER.
     *
     * @endif
     */
    virtual ReturnCode_t reset_component(LightweightRTObject_ptr comp);


    /*!
     * @if jp
     *
     * @brief R|[lg̏Ԃ擾
     *
     * <h3> Description </h3>
     *̃Iy[V͗^ꂽ RTC  LifeCycleState ԂB
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> RTC  AliveԂłȂ΂ȂȂB
     * <li> ^ RTC  ExecutionContext ɏĂȂ΂ȂȂB
     * </ul>
     * @return LifeCycleState::INACTIVE, ACTIVE, ERROR ԂB
     *
     * @else
     *
     * @brief Get component's state
     *
     * <h3> Description </h3>
     * This operation shall report the LifeCycleState of the given participant
     * RTC.
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> The given RTC must be Alive.
     * <li> The given RTC must be a participant in the target ExecutionContext.
     * <li> The LifeCycleState returned by this operation shall be one of
     *      LifeCycleState::INACTIVE, ACTIVE, or ERROR.
     * </ul>
     * @return The LifeCycleState returned by this operation shall be one of
     *         LifeCycleState::INACTIVE, ACTIVE, or ERROR.
     * @endif
     */
    virtual LifeCycleState get_component_state(LightweightRTObject_ptr comp);


    /*!
     * @if jp
     *
     * @brief ExecutionKind 擾
     *
     * <h3> Description </h3>
     * ̃Iy[V ExecutionContext  ExecutionKind ԂB
     *
     * <h3> ExecutionKind </h3>
     * ExecutionKind 񋓎q ExecutionContext ɑ Execution 
     * Z}eBNX (OMG RTC Specification section 2.3 Q) `B
     * <table>
     * <tr><td> PERIODIC </td>
     *     <td> ̃^Cv ExecutionContext ɑ RTC 
     *          periodic sampled data Z}eBNXɏ]Ďs
     *          B(OMG RTC Specification section 2.3.1 Q)</td>
     * </tr>
     * <tr><td> EVENT_DRIVEN </td>
     *     <td> ̃^Cv ExecutionContext ɑ RTC 
     *          stimulus response Z}eBNXɏ]ĎsB
     *          (OMG RTC Specification section 2.3.2 Q) </td>
     * </tr>
     * <tr><td> OTHER </td>
     *     <td> ̃^Cv ExecutionContext ɑ RTC 
     *          ̎sZ}eBNX͎҂CӂɎłA
     *          dl͂Ƃɒ`ĂȂB</td>
     * </tr>
     * </table>
     *
     * @else
     *
     * @brief Get the ExecutionKind
     *
     * <h3> Description </h3>
     * This operation shall report the execution kind of the execution context.
     *
     * <h3> ExecutionKind </h3>
     * The ExecutionKind enumeration defines the execution semantics
     * (see OMG RTC Specification section 2.3) of the RTCs that participate
     * in an execution context.
     * <table>
     * <tr><td> PERIODIC </td>
     *     <td> The participant RTCs are executing according to
     *          periodic sampled data semantics
     *          (see OMG RTC Specification section 2.3.1). </td>
     * </tr>
     * <tr><td> EVENT_DRIVEN </td>
     *     <td>  The participant RTCs are executing according to
     *           stimulus response semantics
     *           (see OMG RTC Specification section 2.3.2).</td>
     * </tr>
     * <tr><td> OTHER </td>
     *     <td> The participant RTCs are executing according to some
     *          semantics not defined by this specification.</td>
     * </tr>
     * </table>
     * @endif
     */
    virtual ExecutionKind get_kind();


    /*!
     * @if jp
     *
     * @brief R|[lgǉ
     *
     * <h3> Description </h3>
     * ̃Iy[V͗^ꂽRTCExecutionContextɏB
     *
     * <h3> Semantics </h3>
     * Vɒǉꂽ@RTC ͂͂߂́@Inactive ԂɂB
     * ExecutionKind  PERIODIC ȂΈ index ͂ RTC s
     * Ԃɂ\[gꂽʒuw肷B
     * ȊOł́Aˑł薳邩ȂB
     *
     * <h3> Constraints </h3>
     * <ul>
     * <li> ExecutionKind  PERIODIC ̏ꍇ, RTC  data flow participant
     *      (OMG RTC Specification section 2.3.1.2 Q) łȂ΂ȂȂB
     *      ȊÓÃIy[V PRECONDITION_NOT_MET
     *      ԂsB.
     * </ul>
     * @return OK or PRECONDITION_NOT_ME Ԃ
     *
     * @else
     *
     * @brief Add a component
     *
     * The operation causes the given RTC to begin participating in the
     * execution context.
     *
     * The newly added RTC will begin in the Inactive state.
     * If the ExecutionKind is PERIODIC, the index represents the sorted order
     * in which the RTC is to be executed. Otherwise, the meaning of the index
     * is implementation-defined and may be ignored.
     * <p>
     * <h3> Constraints </h3>
     * <ul>
     * <li> If the ExecutionKind is PERIODIC, the RTC must be a data flow
     *      participant (see OMG RTC Specification section 2.3.1.2).
     *      Otherwise, this operation will fail with PRECONDITION_NOT_MET.
     * </ul>
     * @return OK or PRECONDITION_NOT_ME would be returned.
     * </p>
     * @endif
     */
    virtual ReturnCode_t add_component(LightweightRTObject_ptr comp);

    /*!
     * @if jp
     *
     * @brief R|[lgR|[lgXg폜
     * 
     * <h3> Description </h3>
     * ̃Iy[V͏Ă RTC 疕B
     * <b> Constraints </b>
     * <ul>
     * <li> ^ꂽ RTC ̃ReLXgɎQĂȂꍇɂ́A
     *      ̃Iy[V BAD_PARAMETER ԂB
     * <li> RTC  ExecutionContext ̏͂ꍇɂ́A
     *      RTC ͔ANeBuȂ΂ȂȂB
     *      ^ꂽ RTC  ExecutionContext ɏĂ̂́A
     *      Active Ԃɂꍇɂ́APRECONDITION_NOT_MET ԂB
     * </ul>
     *
     * @else
     *
     * @brief Remove the component from component list
     *
     * <h3> Description </h3>
     * This operation causes a participant RTC to stop participating in the
     * execution context.
     * 
     * <h3> Constraints </h3>
     * <ul>
     * <li> If the given RTC is not currently participating in the execution
     *      context, this operation shall fail with BAD_PARAMETER.
     * <li> An RTC must be deactivated before it can be removed from an
     *      execution context. If the given RTC is participating in the
     *      execution context but is still in the Active state, this operation
     *      shall fail with PRECONDITION_NOT_MET.
     * </ul>
     * @endif
     */			     
    virtual ReturnCode_t remove_component(LightweightRTObject_ptr comp);


    //============================================================
    // ExecutionContextAdmin interfaces
    //============================================================
    /*!
     * @if jp
     *
     * @brief ExecutionContextProfile 擾
     *
     * <h3> Description </h3>
     * ̃Iy[V ExecutionContext  Profile 擾
     *
     * @else
     *
     * @brief Get the ExecutionContextProfile
     *
     * <h3> Description </h3>
     * This operation provides a profile descriptor for the execution context.
     *
     * @endif
     */
    virtual ExecutionContextProfile* get_profile();


  protected:
    Logger rtclog;
    ExecutionContextProfile m_profile;
    CORBA::Boolean m_running;

    template <class ObjRef>
    struct find_objref
    {
      find_objref(ObjRef ref) : m_ref(ref) {};
      bool operator()(ObjRef ref)
      {
	return ref->_is_equivalent(m_ref);
      }
      ObjRef m_ref;
    };
    /*
    {
                ExecutionKind kind;
                double rate;
                RTObject owner;
                RTCList participants;
                NVList properties;
        };
    */


  };

}; // namespace RTC

#endif // ExecutionContext_h 
