// -*- C++ -*-
/*!
 * @file ModuleManager.h
 * @brief Loadable modules manager class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ModuleManager.h 1243 2009-03-12 09:41:36Z openrtm $
 *
 */

#ifndef ModuleManager_h
#define ModuleManager_h

// STL includes
#include <string>
#include <vector>
#include <map>

// ACE includes
#include <coil/DynamicLib.h>

// RTC includes
#include <rtm/Manager.h>
#include <coil/Properties.h>
#include <rtm/ObjectManager.h>


#define CONFIG_EXT    "manager.modules.config_ext"
#define CONFIG_PATH   "manager.modules.config_path"
#define DETECT_MOD    "manager.modules.detect_loadable"
#define MOD_LOADPTH   "manager.modules.load_path"
#define INITFUNC_SFX  "manager.modules.init_func_suffix"
#define INITFUNC_PFX  "manager.modules.init_func_prefix"
#define ALLOW_ABSPATH "manager.modules.abs_path_allowed"
#define ALLOW_URL     "manager.modules.download_allowed"
#define MOD_DWNDIR    "manager.modules.download_dir"
#define MOD_DELMOD    "manager.modules.download_cleanup"
#define MOD_PRELOAD   "manager.modules.preload"

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  /*!
   * @if jp
   * @class ModuleManager
   * @brief W[}l[WNX
   *
   * W[̃[hAA[hȂǂǗNX
   *
   * @since 0.4.0
   *
   * @else
   * @class ModuleManager
   * @brief ModuleManager class
   *
   * This is a class to manage for loading and unloading modules.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ModuleManager
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     * ݒ肳ꂽ Property IuWFNg̏ɏsB
     *
     * @param prop pvpeB
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor.
     * Initialize based on information in the set Property object.
     *
     * @param prop Properties for initialization
     *
     * @endif
     */
    ModuleManager(coil::Properties& prop);
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    ~ModuleManager(void);
    
    /*!
     * @if jp
     * @brief t@CEI[vsOp\
     * @else
     * @brief Structure for exception handling when file open is failed
     * @endif
     */
    struct Error
    {
      Error(const std::string& _reason)
	: reason(_reason) {}
      std::string reason;
    };
    
    /*!
     * @if jp
     * @brief Cw胂W[sOp\
     * @else
     * @brief Structure for exception handling of unimplemented part
     *        and specified module missing
     * @endif
     */
    struct NotFound
    {
      NotFound(const std::string& _name)
	: name(_name) {}
      std::string name;
    };
    
    /*!
     * @if jp
     * @brief wt@CsOp\
     * @else
     * @brief Structure for exception handling when specified file
     *        cannot be found
     * @endif
     */
    struct FileNotFound
      : public NotFound
    {
      FileNotFound(const std::string& _name)
	: NotFound(_name) {}
    };
    
    /*!
     * @if jp
     * @brief w胂W[sOp\
     * @else
     * @brief Structure for exception handling when specified module
     *        cannot be found
     * @endif
     */
    struct ModuleNotFound
      : public NotFound
    {
      ModuleNotFound(const std::string& _name)
	: NotFound(_name) {}
    };
    
    /*!
     * @if jp
     * @brief wV{sOp\
     * @else
     * @brief Structure for exception handling when specified symbol
     *        cannot be found
     * @endif
     */
    struct SymbolNotFound
      : public NotFound
    {
      SymbolNotFound(const std::string& _name)
	: NotFound(_name) {}
    };
    
    /*!
     * @if jp
     * @brief w葀֎~Op\
     * @else
     * @brief Structure for exception handling when specified
     *        operation cannot be allowed.
     * @endif
     */
    struct NotAllowedOperation
      : public Error
    {
      NotAllowedOperation(const std::string& _reason)
	: Error(_reason) {}
    };
    
    /*!
     * @if jp
     * @brief wsOp\
     * @else
     * @brief Structure for exception handling when specified
     *        argument is invalid.
     * @endif
     */
    struct InvalidArguments
      : public Error
    {
      InvalidArguments(const std::string& _reason)
	: Error(_reason) {}
    };
    
    /*!
     * @if jp
     * @brief w葀sOp\
     * @else
     * @brief Structure for exception handling when specified
     *        operation is invalid.
     * @endif
     */
    struct InvalidOperation
      : public Error
    {
      InvalidOperation(const std::string& _reason)
	: Error(_reason) {}
    };
    typedef void (*ModuleInitFunc)(Manager*);
    
    /*!
     * @if jp
     *
     * @brief W[̃[h
     *
     * file_name DLL ͋LCuƂă[hB
     * file_name ͊̃[hpX (manager.modules.load_path) ɑ΂
     * ΃pXŎw肷B
     *
     * Property manager.modules.abs_path_allowed  yes ̏ꍇA
     * [h郂W[΃pXŎw肷邱ƂłB<br>
     * Property manager.modules.download_allowed  yes ̏ꍇA
     * [h郂W[URLŎw肷邱ƂłB
     *
     * file_name ͐΃pXŎw肷邱ƂłB
     * manager.modules.allowAbsolutePath  no ̏ꍇA
     * ̃W[[hpXAfile_name ̃W[T[hB
     * 
     * @param file_name [hΏۃW[
     *
     * @return w肵[hΏۃW[
     *
     * @else
     *
     * @brief Load the module
     *
     * Load file_name as DLL or a shared liblary.
     * The file_name is specified by the relative path to default load
     * path (manager.modules.load_path).
     *
     * If Property manager.modules.abs_path_allowed is yes,
     * the load module can be specified by the absolute path.<br>
     * If Property manager.modules.download_allowed is yes,
     * the load module can be specified with URL.
     *
     * The file_name can be specified by the absolute path.
     * If manager.modules.allowAbsolutePath is no, module of file_name
     * will be searched from the default module load path and loaded.
     * 
     * @param file_name The target module name for the loading
     *
     * @return Name of module for the specified load
     *
     * @endif
     */
    std::string load(const std::string& file_name);
    
    /*!
     * @if jp
     *
     * @brief W[̃[hA
     *
     * w肵t@CDLL ͋LCuƂă[hƂƂɁA
     * w肵pIy[VsB
     * 
     * @param file_name [hΏۃW[
     * @param init_func pIy[V
     *
     * @return w肵[hΏۃW[
     *
     * @else
     *
     * @brief Load and intialize the module
     *
     * Load the specified file as DLL or a shared library, and execute operation
     * for specified initialization.
     * 
     * @param file_name The target module name for the loading
     * @param init_func Operation for initialization
     *
     * @return Name of module for the specified load
     *
     * @endif
     */
    std::string load(const std::string& file_name, const std::string& init_func);
    
    /*!
     * @if jp
     * @brief W[̃A[h
     *
     * w肵[hς݃W[N[YAA[hB
     *
     * @param file_name A[hΏۃW[
     *
     * @else
     * @brief Unload the module
     *
     * Close and unload the specified module that has been loaded.
     *
     * @param file_name Name of module for the unloading
     *
     * @endif
     */
    void unload(const std::string& file_name);
    
    /*!
     * @if jp
     * @brief SW[̃A[h
     *
     * SẴ[hς݃W[A[hB
     *
     * @else
     * @brief Unload all modules
     *
     * Unload all modules that have been loaded.
     *
     * @endif
     */
    void unloadAll();
    
    /*!
     * @if jp
     * @brief W[̃V{̎Q
     * @else
     * @brief Refer to the symbol of the module
     * @endif
     */
    void* symbol(const std::string& file_name, const std::string& func_name)
      throw (ModuleNotFound, SymbolNotFound);
    
    /*!
     * @if jp
     * @brief W[[hpXw肷
     * 
     * W[[hɑΏۃW[pXw肷B
     *
     * @param load_path W[ΏۃpXXg
     *
     * @else
     * @brief Set the module load path
     * 
     * Specify searching path to find the target module when loading module.
     *
     * @param load_path List of module search path
     *
     * @endif
     */
    void setLoadpath(const std::vector<std::string>& load_path);
    
    /*!
     * @if jp
     * @brief W[[hpX擾
     * 
     * ݒ肳Ă郂W[ΏۃpXXg擾B
     * 
     * @return load_path W[ΏۃpXXg
     *
     * @else
     * @brief Get the module load path
     * 
     * Get the search path of the set module.
     * 
     * @return load_path List of module search path
     *
     * @endif
     */
    inline std::vector<std::string> getLoadPath()
    {
      return m_loadPath;
    }
    
    /*!
     * @if jp
     * @brief W[[hpXǉ
     * 
     * w肳ꂽpXXgΏۃpXXgɒǉB
     * 
     * @return load_path ǉW[ΏۃpXXg
     *
     * @else
     * @brief Add the module load path
     * 
     * Add specified path list to search path list.
     * 
     * @return load_path List of additional module search path
     *
     * @endif
     */
    void addLoadpath(const std::vector<std::string>& load_path);
    
    /*!
     * @if jp
     * @brief [hς݂̃W[Xg擾
     *
     * Ƀ[hς݂̃W[Xg擾B
     *
     * @return [hς݃W[Xg
     *
     * @else
     * @brief Get the module list that has been loaded
     *
     * Get the module list that has been loaded.
     *
     * @return List of module that has been loaded
     *
     * @endif
     */
    std::vector<coil::Properties> getLoadedModules();
    
    /*!
     * @if jp
     * @brief [h\W[Xg擾
     *
     * [h\ȃW[̃Xg擾B
     * ()
     *
     * @return [h\W[Xg
     *
     * @else
     * @brief Get the loadable module list
     *
     * Get the loadable module list (not implemented).
     *
     * @return Loadable module list
     *
     * @endif
     */
    std::vector<coil::Properties> getLoadableModules();
    
    /*!
     * @if jp
     * @brief W[̐΃pXw苖
     *
     * [hΏۃW[̐΃pXw悤ɐݒ肷B
     *
     * @else
     * @brief Allow absolute path when specify module path
     *
     * Set to allow the absolute path when specify the module for the load.
     *
     * @endif
     */
    inline void allowAbsolutePath()
    {
      m_absoluteAllowed = true;
    }
    
    /*!
     * @if jp
     * @brief W[̐΃pXw֎~
     *
     * [hΏۃW[̐΃pXw֎~悤ɐݒ肷B
     *
     * @else
     * @brief Disallow absolute path when specify module path
     *
     * Set to disallow the absolute path when specify the module for the load.
     *
     * @endif
     */
    inline void disallowAbsolutePath()
    {
      m_absoluteAllowed = false;
    }
    
    /*!
     * @if jp
     * @brief W[URLw苖
     *
     * [hΏۃW[URLwB
     * {ݒ肪ĂꍇAW[_E[hă[h邱Ƃ
     * B
     *
     * @else
     * @brief Allow URL when specify module path
     *
     * Allow URL when specify module for the load.
     * When this setup is allowed, downloading and loading the module will
     * be allowed.
     *
     * @endif
     */
    inline void allowModuleDownload()
    {
      m_downloadAllowed = true;
    }
    
    /*!
     * @if jp
     * @brief W[URLw֎~
     *
     * [hΏۃW[URLw֎~B
     *
     * @else
     * @brief Disallow URL when specify module path
     *
     * Disallow URL when specify module for the load.
     *
     * @endif
     */
    inline void disallowModuleDownload()
    {
      m_downloadAllowed = false;
    }
    
    /*!
     * @if jp
     * @brief LoadPath ̃t@Č
     * 
     * w肳ꂽpXɁAw肳ꂽt@C݂邩mFB
     *
     * @param fname Ώۃt@C
     * @param load_path pXXg
     *
     * @return ꂽt@C
     *
     * @else
     * @brief Search the file from the LoadPath
     * 
     * Check whether the specified file exists in the specified path.
     *
     * @param fname Target file name of the search
     * @param load_path Path list for the search
     *
     * @return File name that was found
     *
     * @endif
     */
    std::string findFile(const std::string& fname,
			 const std::vector<std::string>& load_path);
    
    /*!
     * @if jp
     * @brief t@C݂邩ǂ̃`FbN
     *
     * w肳ꂽt@C݂邩mFB
     *
     * @param filename ݊mFΏۃt@C
     *
     * @return t@C݊mF(t@C:trueCȂ:false)
     *
     * @else
     * @brief Check whether the file exists
     *
     * Check whether the specified file exists.
     *
     * @param filename Name of file existence for checking
     *
     * @return File existence result(File existence:true, Else:false)
     *
     * @endif
     */
  bool fileExist(const std::string& filename);
    
    /*!
     * @if jp
     * @brief ֐V{𐶐
     *
     * ֐̖̂gݗĂB
     *
     * @param file_path ΏۃW[
     *
     * @return ֐̑gݗČ
     *
     * @else
     * @brief Create initialization function symbol
     *
     * Assemble names of the initialization functions.
     *
     * @param file_path Name of module for initialization
     *
     * @return Assembly result of initialization function name
     *
     * @endif
     */
    std::string getInitFuncName(const std::string& file_path);
    
  protected:

    struct DLLEntity
    {
      coil::Properties properties;
      coil::DynamicLib dll;
    };
    
    typedef std::vector<std::string>     StringVector;
    typedef StringVector::iterator       StringVectorItr;
    typedef StringVector::const_iterator StringVectorConstItr;
    
    typedef std::vector<DLLEntity>    DllMap;
    typedef DllMap::iterator           DllMapItr;
    typedef DllMap::const_iterator     DllMapConstItr;
    
    /*!
     * @if jp
     * @brief Module Manager vpeB
     * @else
     * @brief Module Manager properties
     * @endif
     */
    coil::Properties& m_properties;
    
    /*!
     * @if jp
     * @brief [hς݃W[Xg
     * @else
     * @brief Module list that has already loaded
     * @endif
     */
    class DllPred
    {
      std::string m_filepath;
    public:
      DllPred(const char* filepath) : m_filepath(filepath) {}
      DllPred(const DLLEntity* dll) : m_filepath(dll->properties["file_path"]) {}
      bool operator()(DLLEntity* dllentity)
      {
        return m_filepath == dllentity->properties.getProperty("file_path");
      }
    };
    //    DllMap m_modules;
    ObjectManager<const char*, DLLEntity, DllPred> m_modules;
    
    /*!
     * @if jp
     * @brief W[E[hEpXEXg
     * @else
     * @brief Module load path list
     * @endif
     */
    StringVector m_loadPath;
    /*!
     * @if jp
     * @brief RtBM[VEpXEXg
     * @else
     * @brief Configuration path list
     * @endif
     */
    StringVector m_configPath;
    /*!
     * @if jp
     * @brief W[URLw苖tO
     * @else
     * @brief Flag of URL when specify module for the load.
     * @endif
     */
    bool m_downloadAllowed;
    /*!
     * @if jp
     * @brief W[΃pXw苖tO
     * @else
     * @brief Flag of absolute path when specify module for the load.
     * @endif
     */
    bool m_absoluteAllowed;
    
    /*!
     * @if jp
     * @brief s֐TtBbNX
     * @else
     * @brief Initial execution function suffix
     * @endif
     */
    std::string m_initFuncSuffix;
    /*!
     * @if jp
     * @brief s֐vtBbNX
     * @else
     * @brief Initial execution function prefix
     * @endif
     */
    std::string m_initFuncPrefix;


    class UnloadPred
    {
    public:
      UnloadPred(){}
      void operator()(DLLEntity* dll)
      {
        dll->dll.close();
        delete dll;
      }
    };

  };   // class ModuleManager
};     // namespace RTC  

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // ModuleManager_h
