// -*- C++ -*-
/*!
 * @file NumberingPolicy.h
 * @brief Object numbering policy class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: NumberingPolicy.h 1225 2009-02-28 02:30:25Z n-ando $
 *
 */

#ifndef NumberingPolicy_h
#define NumberingPolicy_h

#include <string>
#include <vector>

/*!
 * @if jp
 *
 * @class NumberingPolicy
 * @brief IuWFNgl[~OE|V[(K)ǗpۃNX
 *
 * IuWFNg𐶐ۂ̃l[~OE|V[(K)Ǘ邽߂
 * ۃC^[tF[XNXB
 * ۃNX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
 * - onCreate() : IuWFNg̖̍쐬
 * - onDelete() : IuWFNg폜̖̉
 *
 * @since 0.4.0
 *
 * @else
 * @class NumberingPolicy
 * @brief Abstruct class for naming policy management when creating objects
 *
 * This is the abstract interface class to manage the naming policy when
 * creating objects.
 * Concrete classes must implement the following pure virtual functions.
 * - onCreate() : Create the name when creating objects.
 * - onDelete() : Delete the name when deleting objects.
 *
 * @since 0.4.0
 *
 * @endif
 */
class NumberingPolicy
{
public:
  /*!
   * @if jp
   * @brief IuWFNgOp\
   * @else
   * @brief The structures for exception handling when object was not
   *        found.
   * @endif
   */
  struct ObjectNotFound {};
  
  /*!
   * @if jp
   *
   * @brief zfXgN^
   * 
   * @else
   *
   * @brief Virtual destractor
   *
   * @endif
   */
  virtual ~NumberingPolicy(void){};
  
  /*!
   * @if jp
   *
   * @brief IuWFNg̖̍쐬
   *
   * IuWFNĝ̖𐶐邽߂̏z֐
   * 
   * @param obj ̐ΏۃIuWFNg
   *
   * @return IuWFNg
   *
   * @else
   *
   * @brief Create the name when creating objects
   *
   * Pure virtual function to create the name when creating objects.
   * 
   * @param obj The target object for the creation
   *
   * @return Name of the created object
   *
   * @endif
   */
  virtual std::string onCreate(void* obj) = 0;
  
  /*!
   * @if jp
   *
   * @brief IuWFNg폜̖̉
   *
   * IuWFNg폜ɖ̂邽߂̏z֐
   * 
   * @param obj ̉ΏۃIuWFNg
   *
   * @else
   *
   * @brief Delete the name when deleting objects
   *
   * Pure virtual function to delete the name when deleting object.
   * 
   * @param obj The target object of the delete
   *
   * @endif
   */
  virtual void onDelete(void* obj) = 0;
};

/*!
 * @if jp
 *
 * @class DefaultNumberingPolicy
 * @brief IuWFNgl[~OE|V[(K)ǗpNX
 *
 * IuWFNg𐶐ۂ̃l[~OE|V[(K)Ǘ邽߂
 * NXB
 *
 * @since 0.4.0
 *
 * @else
 *
 * @class DefaultNumberingPolicy
 * @brief Class for naming policy management when creating objects
 *
 * This is a class to manage the naming policy when creating objects.
 *
 * @since 0.4.0
 *
 * @endif
 */
class DefaultNumberingPolicy
  : public NumberingPolicy 
{
public:
  /*!
   * @if jp
   *
   * @brief RXgN^
   * 
   * RXgN^
   * 
   * @else
   *
   * @brief Constructor
   * 
   * Constructor
   * 
   * @endif
   */
  DefaultNumberingPolicy() : m_num(0) {};
  
  /*!
   * @if jp
   *
   * @brief fXgN^
   * 
   * @else
   *
   * @brief Destractor
   *
   * @endif
   */
  virtual ~DefaultNumberingPolicy(void){};
  
  /*!
   * @if jp
   *
   * @brief IuWFNg̖̍쐬
   *
   * IuWFNĝ̖𐶐B
   * ς݃CX^X̐ɉ̂𐶐B
   * 
   * @param obj ̐ΏۃIuWFNg
   *
   * @return IuWFNg
   *
   * @else
   *
   * @brief Create the name when creating object
   *
   * Create the name when creating object.
   * Create the name corresponding to the number of generated instances.
   * 
   * @param obj The target object for the name creation
   *
   * @return Names of the created object
   *
   * @endif
   */
  virtual std::string onCreate(void* obj);
  
  /*!
   * @if jp
   *
   * @brief IuWFNg폜̖̉
   *
   * IuWFNg폜ɖ̂B
   * IuWFNg폜ɐς݃CX^XZB
   * 
   * @param obj ̉ΏۃIuWFNg
   *
   * @else
   *
   * @brief Delete the name when deleting object
   *
   * Delete the name when deleting object.
   * Substract the generated number of instances when deleting the object.
   * 
   * @param obj The target object for the name delete
   *
   * @endif
   */
  virtual void onDelete(void* obj);
  
protected:
  /*!
   * @if jp
   *
   * @brief IuWFNǧ
   *
   * IuWFNgXgw肳ꂽIuWFNgA
   * YIuWFNgi[Ăꍇɂ̓CfbNXԂB
   * 
   * @param obj ΏۃIuWFNg
   *
   * @return IuWFNgi[CfbNX
   *
   * @else
   *
   * @brief Find the object
   *
   * Find the specified object in the object list and return its index
   * when it is stored.
   * 
   * @param obj The target object for the find
   *
   * @return Object index for storage
   *
   * @endif
   */
  long int find(void* obj);
  
private:
  int m_num;
  std::vector<void*> m_objects;
};
#endif // NumberingPolicy_h
